library(tidyverse)
library(dplyr)
library(car)
library(caret)
library(ggfortify)

#Take data from previouse lab with assigned groups
dataset = read_delim('daSilva.csv',
                     delim = ";",
                     locale=locale(decimal_mark = ","),
                     col_names = TRUE,
                     trim_ws = TRUE)


dataset <- dataset %>%
  select(-c(NDB_No, FdGrp_Cd, NDB_No_1, GmWt_Desc2, GmWt_Desc1))

dataset <- dataset%>%
  select_if(~ sum(is.na(.)) < 10) %>%
  drop_na()

dataset2 <- dataset %>% 
  select(-"00000Shrt_Desc") 

dataset2 <- dataset2%>%
  select(-nearZeroVar(dataset2)) 

dataset <- dataset2 %>%
  #select(-findCorrelation(cor(dataset2), cutoff = 0.6))%>%
  mutate('00000Shrt_Desc' = dataset$`00000Shrt_Desc`) %>%
  select('00000Shrt_Desc', everything())

#------------------------------------PCA------------------------------------------


#create the PCA vectors
model_pca <- prcomp(subset(dataset, select = -c(`00000Shrt_Desc`, clusters_assigned_kmeans, clusters_assigned_hc)), 
                    center = TRUE, 
                    scale = TRUE)

summary(model_pca)



#apply to calculate loadings
loadings_pca <- as_tibble(model_pca$rotation)

#add the scores and information about categories
dataset_pca <- as_tibble(predict(model_pca)) %>%
  mutate(clusters_assigned_hc = factor(dataset$clusters_assigned_hc),
         clusters_assigned_kmeans = factor(dataset$clusters_assigned_kmeans))

#graph
car::scatter3d(x = dataset_pca$PC1,
          y = dataset_pca$PC2,
          z = dataset_pca$PC3,
          main="3D PCA",
          xlab = "PC1",
          ylab = "PC2",
          zlab = "PC3",
          surface = FALSE,
          groups = dataset_pca$clusters_assigned_hc,
          axis.col = c("black","black","black"),
          axis.size = 2,
          sphere.size = 3)
#------------------------
# autoplot(model_pca, #what is the PC model
#          data = dataset_pca, #which data the model is applied to
#          colour = 'clusters_assigned_hc', #can be used to colour points according to some groups
#          size = 3, #how large is the point size
#          loadings = TRUE, #are loadings plotted
#          loadings.colour = 'blue', #what colour are the loadings plotted
#          loadings.label = TRUE, #are the varaible names added to the arrows
#          loadings.label.size = 3 #how large are the datapointson the graph
#          ) +
#   theme_bw() #I use to remove background
#--------------------


ggplot() + 
  geom_point(mapping = aes(x = dataset_pca$PC1, 
                           y = dataset_pca$PC3, 
                           color = dataset_pca$clusters_assigned_hc),
             size = 2,
             alpha = 0.75) + 
  geom_segment(mapping = aes(x = 0,  
                             xend = 10*loadings_pca$PC1, 
                             y = 0, 
                             yend = 10*loadings_pca$PC3), 
               arrow = arrow(),
               color = "gray31") + 
  geom_text(mapping = aes(x = 12*loadings_pca$PC1,
                          y = 12*loadings_pca$PC3,
                          label = colnames(dataset[4:])),
            size = 4,
            color = "black") + 
  labs(x = "PC1", y = "PC3") +
  theme(panel.background = element_blank(),
        axis.ticks = element_blank(),
        axis.text = element_text(size=14, face = "plain", colour = "gray"),
        axis.title = element_text(size=14, face = "plain", colour = "gray"),
        legend.position = "bottom"
  ) 
ggsave('name_of_the_graph.svg')

#-------------------k-means on top of it------------------------------

set.seed(6)
wcss = vector()
for (i in 1:25) wcss[i] = sum(kmeans(dataset_pca %>% select(-c(clusters_assigned_kmeans, clusters_assigned_hc)), i)$withinss)
plot(1:25,
     wcss,
     type = 'b',
     main = paste('The Elbow Method'),
     xlab = 'Number of clusters',
     ylab = 'WCSS')


# Fitting K-Means to the dataset
kmeans_14 = kmeans(x = dataset_pca %>% select(-c(clusters_assigned_kmeans, clusters_assigned_hc)), centers = 14)
y_kmeans = kmeans_14$cluster

dataset_pca<- dataset_pca %>%
  mutate(clusters_assigned_kmeans_PCA = y_kmeans) %>%
  mutate(commodity = dataset$`00000Shrt_Desc`)


# Visualising the clusters
library(cluster)
clusplot(dataset[3:13],
         y_kmeans,
         lines = 0,
         shade = TRUE,
         color = TRUE,
         labels = 2,
         plotchar = FALSE,
         span = TRUE,
         main = paste('Clusters of commodity groups'),
         xlab = 'PC1',
         ylab = 'PC2')